package polyglot.ext.predj.ast;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.util.*;
import polyglot.visit.*;
import polyglot.types.*;
import polyglot.ext.predj.types.*;
import polyglot.ext.predj.types.Predicate;

import polyglot.ext.jl5.types.*;

// a base class for the different kinds of specializer nodes
public abstract class AbstractSpecializerNode_c
    extends BindingOrSpecializerNode_c implements AbstractSpecializerNode {

    protected TypeNode specializer;
    
    public AbstractSpecializerNode_c(Position pos, Expr target,
			     TypeNode specializer) {
	super(pos, target);
	this.specializer = specializer;
    }

    public TypeNode specializer() {
	return specializer;
    }

  public Node typeCheck(TypeChecker tc) throws SemanticException {
	AbstractSpecializerNode_c n =
	    (AbstractSpecializerNode_c) super.typeCheck(tc);

	PredJTypeSystem ts = (PredJTypeSystem) tc.typeSystem();
	
	Expr target = n.target();
	Type targetT = target.type();
	Type erasedTargetT = ts.erasure(targetT);
	Type specT = n.specializer().type();
	Type erasedSpecT = ts.erasure(specT);

	    // specializing on a parameterized type is potentially unsafe
	if (specT instanceof ParameterizedType) {
	    ErrorQueue eq = tc.errorQueue();
	    eq.enqueue(ErrorInfo.WARNING,
		       "Specializing on a parameterized type could lead to "
		       + "unchecked operations.", position());
	}

	    // not allowing primitive types to be specialized
	    // Need to strip off array dimensions if we have an array
	    // to make sure that its ultimate base is not a primitive type.
	Type targetTpossiblyArr = targetT;
	while(targetTpossiblyArr.isArray())
	    targetTpossiblyArr = targetTpossiblyArr.toArray().base();
	    
	if(targetTpossiblyArr.isPrimitive()) {
	    throw new SemanticException("Expressions of primitive type " +
					"and arrays of primitive type " +
					"may not be specialized.",
					position());
	}
	

	if (!ts.isCastValid(targetT, specT)) {
	    throw new SemanticException(
		"An object having the known type of the target cannot be cast to the given specializer.  See the rules for casting conversion in the Java Language Specification.",
		position());
	}

	if(erasedSpecT.equals(erasedTargetT)) {
	    throw new SemanticException(
		"This predicate is always true, because the specializer is equal to the "
		+ "known type of the target.", position());
	}
	
	if(specT.equals(ts.Object())) {
	    throw new SemanticException(
		"This predicate is always true, because the specializer is java.lang.Object.", position());
	}

	if(erasedTargetT.descendsFrom(erasedSpecT)) {
 	    throw new SemanticException(
 		"This predicate is always true, since the known type of "
		+ "the target is a strict subclass of the specializer.",
		position());
	}

	return n;
  }
    
}
