package polyglot.ext.predj.ast;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.util.*;
import polyglot.types.*;
import polyglot.ext.predj.types.PredJTypeSystem;
import polyglot.ext.predj.types.PredJMethodInstance;
import polyglot.ext.predj.types.Predicate;
import polyglot.visit.*;

import polyglot.ext.jl5.ast.JL5MethodDecl_c;
import polyglot.ext.jl5.types.FlagAnnotations;

import java.util.*;

/** An abstract base class for PredJava method declarations. **/
public abstract class PredJAbstractMethodDecl_c extends JL5MethodDecl_c
    implements PredJMethodDecl {


	// is the method part of a dispatcher?
    protected boolean hasMates = false;

	// a number used to give this mate a unique name in the Java
	// translation, if it is a mate.
    protected int mateNumber;
    
    public PredJAbstractMethodDecl_c(Position pos, FlagAnnotations flags,
				     TypeNode returnType, String name,
				     List formals,
				     List throwTypes, Block body,
				     List paramTypes) {
	super(pos, flags, returnType, name, formals, throwTypes, body,
	      paramTypes);
	setParamPosition(formals);
    }
    
    private void setParamPosition(List formals) {
	    // set the parameter position of each formal
	Iterator i = formals.iterator();
	int count = 0;
	while(i.hasNext()) {
	    PredJFormal form = (PredJFormal) (i.next());
	    form.setParamPosition(count);
	    count++;
	}
    }

    
	// does this method have a predicate?
    public boolean hasPredicate() {
	return false;
    }
    
	// set this mate's unique mate number, used to give it
	// a unique name in the Java translation
    public void setMateNumber(int i) {
	mateNumber = i;
    }

    public int mateNumber() {
	return mateNumber;
    }


    public void setHasMates() {
	hasMates = true;
    }
    
	// does this method have any dispatcher mates?
    public boolean hasMates() {
	return hasMates;
    }	

	// is the given method a dispatcher mate of this one?
    public boolean hasDispatcherMate(PredJMethodDecl m) {
	PredJMethodInstance thisMI = (PredJMethodInstance) methodInstance();
	PredJMethodInstance otherMI = (PredJMethodInstance) m.methodInstance();
	
	return thisMI.hasDispatcherMate(otherMI);
    }
    
	// this method already exists in MethodDecl, but it is protected.
	// making it public allows it to be accessed by PredJTranslator.
    public MethodInstance makeMethodInstance(ClassType ct, TypeSystem ts)
	throws SemanticException {
	return super.makeMethodInstance(ct, ts);
    }
}
