package polyglot.ext.predj.ast;

import polyglot.ast.*;
import polyglot.visit.*;
import polyglot.ext.jl.ast.*;
import polyglot.util.*;
import polyglot.types.*;
import polyglot.ext.predj.types.*;
import polyglot.ext.predj.types.Predicate;
import polyglot.ext.predj.util.toPredicate;
import polyglot.ext.predj.util.checkPredicateRestrictions;

public class PredicateNode_c extends Node_c
    implements PredicateNode {

    protected Expr predicateExpr;
    
    public PredicateNode_c(Position pos, Expr predicateExpr) {
	super(pos);
	this.predicateExpr = predicateExpr;
    }

    public Expr predicateExpr() {
	return predicateExpr;
    }

    public Node visitChildren(NodeVisitor v) {
	Expr predicateExpr = (Expr) visitChild(this.predicateExpr, v);
	return reconstruct(predicateExpr);
    }

    protected PredicateNode_c reconstruct(Expr predicateExpr) {
	if (predicateExpr != this.predicateExpr) {
	    PredicateNode_c n = (PredicateNode_c) copy();
	    n.predicateExpr = predicateExpr;
	    return n;
	}
	return this;
    }
    
    public Predicate toPredicate(PredJTypeSystem ts) {
	return toPredicate.toPredicate(predicateExpr, ts);
    }

    public Node typeCheck(TypeChecker tc) throws SemanticException {
	PredicateNode n = (PredicateNode) super.typeCheck(tc);

	Expr predExpr = n.predicateExpr();

	    // make sure the predicateExpr has type boolean
	if (!(predExpr.type().isBoolean())) {
	    throw new SemanticException("A method predicate must have type "
				        + "boolean.", position());
	}
	    // make sure that the restrictions on array accesses are met
	checkPredicateRestrictions.checkPredicateRestrictions(predicateExpr);
	return n;
    } 
}
