package polyglot.ext.predj.util;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.ext.predj.ast.*;
import polyglot.types.*;
import polyglot.ext.predj.types.*;

// rewrite PredJ expressions into ordinary Java expressions

public class toJavaExpr {

    public static Receiver toJavaExpr(
	Receiver r, NodeFactory nf, PredJTypeSystem ts) {
	if (r instanceof Local) {
	    return r;
	} else if (r instanceof Field) {
	    Field f = (Field) r;
// we assume for now that this is a target, i.e. it is a simple path
// to a field (rather than the prefix being some arbitrary expression)
	    return f.target(toJavaExpr(f.target(), nf, ts));
	} else if (r instanceof Call) {
	    Call c = (Call) r;
	    return c.target(toJavaExpr(c.target(), nf, ts));
	} else if (r instanceof Cast) {
	    Cast cast = (Cast) r;
	    return cast.expr((Expr) toJavaExpr(cast.expr(), nf, ts));
	} else if ((r instanceof Lit) || (r instanceof Special)) {
	    return r;
	} else if (r instanceof AbstractSpecializerNode) {
	    AbstractSpecializerNode sn = (AbstractSpecializerNode) r;
	    Expr target = (Expr) toJavaExpr(sn.target(), nf, ts);

		// If the expression being instanceof-tested is a cast
		// of another expr, remove the unnecessary cast. Else,
		// ordered predicate dispatchs can generate illegal
		// expressions of the form "((I1) o) instanceof I2",
		// where I1 and I2 have conflicting methods. Removing
		// the cast to I1 is safe as long as the possible cast
		// exception is not intended behavior.  At the time of
		// writing the while looping is not necessary, but it
		// could be needed at some point.
	    while (target instanceof Cast) {
		target = ((Cast) target).expr();
	    }

	    TypeNode spec = sn.specializer();
	    return nf.Instanceof(null, target,
				     // need to erase the specializer
				 nf.CanonicalTypeNode(
				     null, ts.erasure(spec.type())));
	} else if (r instanceof BindingNode) {
	    return nf.BooleanLit(null, true);
	} else if (r instanceof Binary) {
	    Binary b = (Binary) r;
	    return nf.Binary(null,
			     (Expr) toJavaExpr(b.left(), nf, ts), b.operator(),
			     (Expr) toJavaExpr(b.right(), nf, ts));
	} else if (r instanceof Unary) {
	    Unary u = (Unary) r;
	    return nf.Unary(null,
			    u.operator(), (Expr) toJavaExpr(u.expr(), nf, ts));
	} else if (r instanceof TypeNode) {
	    return r;
	} else if (r instanceof ArrayAccess) {
	    ArrayAccess a = (ArrayAccess) r;
	    return nf.ArrayAccess(null,
				  (Expr) toJavaExpr(a.array(), nf, ts),
				  (Expr) toJavaExpr(a.index(), nf, ts));
	} else {
	    throw new RuntimeException("Don't know how to convert " + r +
				       " to a Java expression.");
	}
    }
}
