package polyglot.ext.predj.util;

import polyglot.ast.*;
import polyglot.ext.jl.ast.*;
import polyglot.ext.predj.ast.*;
import polyglot.types.*;
import polyglot.ext.predj.types.*;

import java.util.*;

// produce the type information (Predicate) for an expression (Expr)
// appearing in a predicate.  we assume the expression has already been
// canonicalized (see visit/PredicateCanonicalizer.java).

public class toPredicate {
/*	// Create a unique identifier for array types by using the target's
	// name and type.
    public static String mangleArray(String target, String type) {
	return target + "$" + type.replace("[]", "$Arr");
    }
*/	
    public static Predicate toPredicate(Expr e, PredJTypeSystem ts) {
	if (e instanceof ArrayAccess) {
	    ArrayAccess a = (ArrayAccess) e;
	    return ts.createPredicateArrayAccess(toPredicate(a.array(), ts),
						 toPredicate(a.index(), ts),
						 a.type());	
	} else if (e instanceof Local) {
	    Local l = (Local) e;
		//if (l.type().isArray()) 
		//return ts.createPredicateTarget(mangleArray(l.toString(), l.type().toString()),
		//					    l.type());
	
	    return ts.createPredicateTarget(l.toString(), l.type());
	} else if (e instanceof Field) {
	    Field f = (Field) e;
// we assume for now that this is a target, i.e. it is a simple path
// to a field (rather than the prefix being some arbitrary expression)
	    return ts.createPredicateTarget(canonicalTargetName.
					    canonicalTargetName(f), f.type());
	} else if (e instanceof Call) {
	    Call c = (Call) e;
	    MethodInstance mi = c.methodInstance();
	    List args = c.arguments(), argTypes = mi.formalTypes();
	    Vector predicateArgs = new Vector();
	    String mangledName = "";
	    Iterator i = argTypes.iterator();
	    while (i.hasNext())
		mangledName += "." + i.next();
	    if (mi.flags().contains(ts.Static())) {
		mangledName = mi.name() + "." + mi.container().toString()
		    + mangledName;
	    }
	    else {
		Vector augmentedArgTypes = new Vector();
		augmentedArgTypes.add(mi.container());
		augmentedArgTypes.addAll(argTypes);
		argTypes = augmentedArgTypes;
		predicateArgs.add(toPredicate((Expr) c.target(), ts));
		List overrides = mi.overrides();
		int baseClassIdx = overrides.size() - 1;
		String baseClassName = 
		    ((MethodInstance)overrides.get(baseClassIdx)).
		    container().toString();
		mangledName = mi.name() + "." + baseClassName + mangledName;
	    }
	    i = args.iterator();
	    while (i.hasNext())
		predicateArgs.add(toPredicate((Expr) i.next(), ts));
	    return ts.createPredicateCall(mangledName, predicateArgs,
					  c.type(), argTypes);
	} else if (e instanceof Lit) {
	    Lit lit = (Lit) e;
	    return ts.createPredicateLit(lit.constantValue(), lit.type());
	} else if (e instanceof AbstractSpecializerNode) {
	    AbstractSpecializerNode sn = (AbstractSpecializerNode) e;
	    return ts.createSpecializer(toPredicate(sn.target(), ts),
					sn.specializer().type(),
					sn.polarity());
	} else if (e instanceof BindingNode) {
	    return ts.truePredicate();
	} else if (e instanceof Binary) {
	    Binary b = (Binary) e;
	    return ts.createPredicateBinary(
		toPredicate(b.left(), ts), b.operator(),
		toPredicate(b.right(), ts), b.type());
	} else if (e instanceof Unary) {
	    Unary u = (Unary) e;
	    return ts.createPredicateUnary(
		u.operator(), toPredicate(u.expr(), ts), u.type());
	} else if (e instanceof Special) {
	    Special s = (Special) e;
	    return ts.createPredicateSpecial(s.kind(), s.type());
	} else if (e instanceof Cast) {
	    Cast cast = (Cast) e;
		//ignore casts
	    return toPredicate(cast.expr(), ts);
	} else {
	    throw new RuntimeException(
		"toPredicate called on an unexpected expression " + e);
	}
    }
}
