
package polyglot.ext.predj.visit;

import polyglot.visit.NodeVisitor;
import polyglot.ast.*;
import polyglot.ext.predj.ast.*;
import polyglot.ext.predj.types.*;

import java.util.*;

// a pass that extracts the new variable bindings for a method from its
// associated predicate

public class BindingsExtractor extends NodeVisitor {

	// keeps track of the specialized types of formal parameters
	// in a predicate;
	// a set of SpecializerNodes
    private Set formalBindings = new HashSet();
	// keeps track of any new variable bindings made in a predicate;
	// a list of BindingSpecializerNodes
	// it's a list so that the bindings can be declared in the proper
	// order, since later ones may refer to earlier ones
    private List localBindings = new LinkedList();

    public Node leave(Node old, Node n, NodeVisitor v) {
	if (n instanceof PredJPredMethodDecl) {
	    PredJPredMethodDecl node =
		((PredJPredMethodDecl)n).bindings(formalBindings,
						  localBindings);
	    // reset the formal bindings for the next method
	    formalBindings = new HashSet();
	    localBindings = new LinkedList();
	    return node;
	} else if (n instanceof BindingNode) {
		// add var bindings.
	    if (((BindingNode)n).polarity()) {
		localBindings.add(n);
	    }
	    return n;
	} else if (n instanceof SpecializerNode) {
		// treat formals specially
	    SpecializerNode sn = (SpecializerNode) n;
	    if (sn.polarity() && sn.specializeType()) {
		formalBindings.add(sn);
	    }
	    return sn;
	} else
	    return super.leave(old, n, v);
    }
}
